@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 #  This file is part of Bioconvert software
 #
@@ -11,7 +10,7 @@
 #  documentation: http://bioconvert.readthedocs.io
 #
 ##############################################################################
-"""Convert :term:`BCF` format to :term:`WIGGLE` formats"""
+"""Convert :term:`BCF` format to :term:`WIGGLE` format"""
 from bioconvert import ConvBase
 import colorlog
 
@@ -23,9 +22,13 @@ __all__ = ["BCF2WIGGLE"]
 
 
 class BCF2WIGGLE(ConvBase):
-    """Convert sorted :term:`BCF` file into :term:`WIGGLE` file 
+    """Convert sorted :term:`BCF` file into :term:`WIGGLE` file
+
+    Methods available are based on wiggletools [WIGGLETOOLS]_.
 
     """
+
+    #: Default value
     _default_method = "wiggletools"
 
     def __init__(self, infile, outfile):
@@ -33,13 +36,12 @@ class BCF2WIGGLE(ConvBase):
         :param str infile: The path to the input BCF file. **It must be sorted**.
         :param str outfile: The path to the output file
         """
-        super().__init__(infile, outfile)
+        super(BCF2WIGGLE, self).__init__(infile, outfile)
 
     @requires("wiggletools")
     def _method_wiggletools(self, *args, **kwargs):
-        """
+        """Conversion using wiggletools
 
-        """
+        `wiggletools documentation <https://github.com/Ensembl/WiggleTools>`_"""
         cmd = "wiggletools {} > {}".format(self.infile, self.outfile)
         self.execute(cmd)
-
