@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 #  This file is part of Bioconvert software
 #
@@ -11,11 +10,10 @@
 #  documentation: http://bioconvert.readthedocs.io
 #
 ##############################################################################
-"""Convert :term:`BEDGRAPH` format to :term:`WIGGLE` formats"""
+"""Convert :term:`BEDGRAPH` format to :term:`WIGGLE` format"""
 from bioconvert import ConvBase
-import colorlog
-
 from bioconvert.core.decorators import requires
+import colorlog
 
 _log = colorlog.getLogger(__name__)
 
@@ -23,9 +21,13 @@ __all__ = ["BEDGRAPH2WIGGLE"]
 
 
 class BEDGRAPH2WIGGLE(ConvBase):
-    """Convert sorted :term:`BEDGRAPH` file into :term:`WIGGLE` file 
+    """Convert sorted :term:`BEDGRAPH` file into :term:`WIGGLE` file
+
+    Methods available are based on wiggletools [WIGGLETOOLS]_.
 
     """
+
+    #: Default value
     _default_method = "wiggletools"
 
     def __init__(self, infile, outfile):
@@ -33,16 +35,17 @@ class BEDGRAPH2WIGGLE(ConvBase):
         :param str infile: The path to the input BEDGRAPH file. **It must be sorted**.
         :param str outfile: The path to the output file
         """
-        super().__init__(infile, outfile)
+        super(BEDGRAPH2WIGGLE, self).__init__(infile, outfile)
 
     @requires("wiggletools")
     def _method_wiggletools(self, *args, **kwargs):
-        """wiggletools based method
+        """wiggletools based method. Extension must be .bg.
 
-        Extension must be .bg
-        """
+
+        `wiggletools documentation <https://github.com/Ensembl/WiggleTools>`_"""
         if self.infile.endswith(".bg") is False:
-            raise ValueError("The input bedgraph file must have the .bg extension (wiggletools requirements)")
+            raise ValueError(
+                "The input bedgraph file must have the .bg extension (wiggletools requirements)"
+            )
         cmd = "wiggletools {} > {}".format(self.infile, self.outfile)
         self.execute(cmd)
-
