@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 ###########################################################################
 # Bioconvert is a project to facilitate the interconversion               #
 # of life science data from one format to another.                        #
@@ -22,6 +20,7 @@
 # along with this program (COPYING file).                                 #
 # If not, see <http://www.gnu.org/licenses/>.                             #
 ###########################################################################
+"""Main factory of Bioconvert"""
 import copy
 import time
 import abc
@@ -43,81 +42,63 @@ import colorlog
 
 import bioconvert
 
-_log = colorlog.getLogger(__name__)
-
 from bioconvert.core.benchmark import Benchmark
+from bioconvert.core import extensions
+
 from bioconvert.core.utils import generate_outfile_name
 from bioconvert import logger
+from . import BioconvertError
+
+
+_log = colorlog.getLogger(__name__)
 
 
 class ConvMeta(abc.ABCMeta):
-    """
-    This metaclass checks that the converter classes have
+    """This metaclass checks that the converter classes have
 
        * an attribute input_ext
        * an attribute output_ext
 
-    This is an abstract class used by :class:`ConvBase` class. For developers
+    This is a meta class used by :class:`ConvBase` class. For developers
     only.
     """
 
     @classmethod
-    def split_converter_to_extensions(cls, converter_name: str):
+    def split_converter_to_format(cls, converter_name: str):
         converter_name = converter_name.replace("_to_", "2")
-        if '2' not in converter_name:
-            raise TypeError("converter's name '%s' name must follow convention input2output" % converter_name)
+        if "2" not in converter_name:
+            raise TypeError(
+                "converter's name '{}' name must follow convention input2output".format(
+                    converter_name
+                )
+            )
         # for BZ2 2 GZ
         if "22" in converter_name:
-            input_fmt, output_fmt = converter_name.upper().split('22', 1)
+            input_fmt, output_fmt = converter_name.upper().split("22", 1)
             input_fmt += "2"
+            input_fmt = tuple([input_fmt])
+            output_fmt = tuple([output_fmt])
         else:
-            input_fmt, output_fmt = converter_name.upper().split('2', 1)
+            input_fmt, output_fmt = converter_name.upper().split("2", 1)
+            input_fmt = input_fmt.upper().split("_")
+            input_fmt = tuple(input_fmt)
+            output_fmt = output_fmt.upper().split("_")
+            output_fmt = tuple(output_fmt)
+
         return input_fmt, output_fmt
 
+    @classmethod
+    def lower_tuple(cls, format_tuple):
+        format_tuple = [format.lower() for format in format_tuple]
+        return format_tuple
+
     def __init__(cls, name, bases, classdict):
 
         # do not check extension since modules does not require to specify
         # extension anymore
 
-        # def check_ext(ext, io_name):
-        #     """
-        #     Check if the extension is specified correctly.
-        #     I must be a string or a sequence of string, otherwise raise an error
-        #     it should start with a dot. Otherwise fix extension and inject it in the class
-
-        #     :param ext: the value of the class attribute (input|output)_ext
-        #     :type ext: a string or a list, tuple or set of strings
-        #     :param str io_name: the type of extension, 'input' or output'
-        #     :raise TypeError:  if ext is neither a string nor a sequence of strings
-        #     """
-        #     if isinstance(ext, str):
-        #         if not ext.startswith('.'):
-        #             ext = '.' + ext
-        #         setattr(cls, '{}_ext'.format(io_name),  (ext, ))
-        #     elif isinstance(ext, (list, tuple, set)):
-        #         if not all([isinstance(one_ext, str) for one_ext in ext]):
-        #             raise TypeError("each element of the class attribute '{}.{}_ext' "
-        #                             "must be a string".format(cls, io_name))
-        #         else:
-        #             if not all([one_ext.startswith('.') for one_ext in ext]):
-        #                 fixed_ext = []
-        #                 for one_ext in ext:
-        #                     if one_ext.startswith('.'):
-        #                         fixed_ext.append(one_ext)
-        #                     else:
-        #                         fixed_ext.append('.' + one_ext)
-        #                 setattr(cls, '{}_ext'.format(io_name), fixed_ext)
-        #     else:
-        #         import sys
-        #         err = "the class attribute '{}.{}_ext' " \
-        #               "must be specified in the class or subclasses".format(cls.__name__, io_name)
-        #         _log.warning("skip class '{}': {}".format(cls.__name__, err, file=sys.stderr))
-        #         raise TypeError("the class attribute '{}.{}_ext' must be specified "
-        #                         "in the class or subclasses".format(cls.__name__, io_name))
-        #     return True
-
         def is_conversion_method(item):
-            """Return True is method name starts with _method_
+            """Return True if method name starts with _method_
 
             This method is used to keep methods that starts with _method_.
             It uses inspect.getmembers func to list
@@ -127,39 +108,88 @@ class ConvMeta(abc.ABCMeta):
             :return: True if method's name starts with '__method_', False otherwise.
             :rtype: boolean
             """
-            return inspect.isfunction(item) and \
-                    item.__name__.startswith('_method_') and \
-                    item.__name__ != "_method_dummy"
 
-        if name != 'ConvBase':
-            input_fmt, output_fmt = cls.split_converter_to_extensions(name.upper())
-            # modules have no more input_ext and output_ext attributes
-            # input_ext = getattr(cls, 'input_ext')
-            # if check_ext(input_ext, 'input'):
-            #     output_ext = getattr(cls, 'output_ext')
-            #     check_ext(output_ext, 'output')
-            setattr(cls, 'input_fmt', input_fmt)
-            setattr(cls, 'output_fmt', output_fmt)
+            return inspect.isfunction(item) and item.__name__.startswith("_method_")
+
+        if name != "ConvBase":
+            input_fmt, output_fmt = cls.split_converter_to_format(name)
+            setattr(cls, "input_fmt", input_fmt)
+            setattr(cls, "output_fmt", output_fmt)
+
+            if not cls.input_ext:
+                # We add all the extensions for each converter into a list.
+                input_ext = []
+                cls.input_ext = cls.lower_tuple(cls.input_fmt)
+                for format in cls.input_ext:
+                    input_ext.append(tuple(extensions.extensions[format]))
+                # then we turn the list into tuple as output_ext attribute
+                setattr(cls, "input_ext", tuple(input_ext))
+            # if the developer did not specify an output_ext attribute
+            if not cls.output_ext:
+                # We add all the extensions for each converter into a list.
+                output_ext = []
+                cls.output_ext = cls.lower_tuple(cls.output_fmt)
+                for format in cls.output_ext:
+                    output_ext.append(tuple(extensions.extensions[format]))
+                # then we turn the list into tuple as output_ext attribute
+                setattr(cls, "output_ext", tuple(output_ext))
+                # if the key is not in the dictionary return an error message
             available_conv_meth = []
             for name in inspect.getmembers(cls, is_conversion_method):
                 # do not use strip() but split()
                 conv_meth = name[0].split("_method_")[1]
                 is_disabled = getattr(name[1], "is_disabled", None)
                 if is_disabled is None:
-                    _log.debug("converter '{}': method {} is not decorated, we expect it to work all time".format(
-                        cls.__name__,
-                        conv_meth,
-                    ))
+                    _log.debug(
+                        "converter '{}': method {} is not decorated, we expect it to work all time".format(
+                            cls.__name__, conv_meth,
+                        )
+                    )
                     is_disabled = False
                 if not is_disabled:
                     available_conv_meth.append(conv_meth)
                 else:
-                    _log.warning("converter '{}': method {} is not available".format(cls.__name__, conv_meth, ))
-            setattr(cls, 'available_methods', available_conv_meth)
-            _log.debug("class = {}  available_methods = {}".format(cls.__name__, available_conv_meth))
+                    _log.warning(
+                        "converter '{}': method {} is not available".format(
+                            cls.__name__, conv_meth,
+                        )
+                    )
+            setattr(cls, "available_methods", available_conv_meth)
+            _log.debug(
+                "class = {}  available_methods = {}".format(
+                    cls.__name__, available_conv_meth
+                )
+            )
 
 
 class ConvArg(object):
+    """This class can be used to add specific extra arguments to any converter
+
+    For instance, imagine a conversion named **A2B** that requires the
+    user to provide a reference. Then, you may want to provide the
+    `--reference` extra argument. This is possible by adding a class
+    method named get_additional_arguments that will yield instance of
+    this class for each extra argument.
+
+    ::
+
+        @classmethod
+        def get_additional_arguments(cls):
+            yield ConvArg(
+                names="--reference",
+                default=None,
+                help="the referenc"
+            )
+
+    Then, when calling bioconvert as follows,::
+
+        bioconvert A2B --help
+
+    the new argument will be shown in the list of arguments.
+
+
+    """
+
     black_listed_argument_for_argparse = [
         "output_argument",
     ]
@@ -168,10 +198,10 @@ class ConvArg(object):
         if isinstance(names, list):
             self.args_for_sub_parser = names
         else:
-            self.args_for_sub_parser = [names, ]
-        self.kwargs_for_sub_parser = {
-            'help': help
-        }
+            self.args_for_sub_parser = [
+                names,
+            ]
+        self.kwargs_for_sub_parser = {"help": help}
         self.kwargs_for_sub_parser.update(kwargs)
 
     def add_to_sub_parser(self, sub_parser):
@@ -186,15 +216,15 @@ class ConvArg(object):
 
 
 class ConvBase(metaclass=ConvMeta):
-    """ base class for all converters.
+    """Base class for all converters.
 
-    To build a new converter create a new class which inherits from
+    To build a new converter, create a new class which inherits from
     :class:`ConvBase` and implement method that performs the conversion.
-    The name of the converter method must start with _method_.
+    The name of the converter method must start with ``_method_``.
 
     For instance: ::
 
-        class Fastq2Fasta(ConvBase):
+        class FASTQ2FASTA(ConvBase):
 
             def _method_python(self, *args, **kwargs):
                 # include your code here. You can use the infile and outfile
@@ -203,6 +233,7 @@ class ConvBase(metaclass=ConvMeta):
                 self.outfile
 
     """
+
     # specify the extensions of the input file, can be a sequence (must be
     # overridden in subclasses)
     input_ext = None
@@ -210,8 +241,23 @@ class ConvBase(metaclass=ConvMeta):
     # specify the extensions of the output file, can be a sequence (must be
     # overridden in subclasses)
     output_ext = None
+
+    # list available methods
+    available_methods = []
+
+    # default method should be provided
     _default_method = None
     _is_compressor = False
+    # Can be overriden and if True, new argument --thread is added automatically
+    _threading = False
+    _extra_arguments = ""
+
+    # Used for the benchmarking only
+    others = {}
+
+    # threads to be used by default if argument is required in a method
+    # this will be overriden if _threading set to True and therefore --threads
+    # set by the user. It is feed back into Bioconvert class
     threads = cpu_count()
 
     def __init__(self, infile, outfile):
@@ -220,17 +266,20 @@ class ConvBase(metaclass=ConvMeta):
         :param str infile: the path of the input file.
         :param str outfile: the path of The output file
         """
-        if not outfile:
-            outfile = generate_outfile_name(infile, self.output_ext[0])
 
         self.infile = infile
         self.outfile = outfile
-        self._execute_mode = "shell" #"subprocess"  # set to shell to call shell() method
+
+        # execute mode can be shell or subprocess.
+        self._execute_mode = "shell"
+
+        # The logger to be set to INFO, DEBUG, WARNING, ERROR, CRITICAL
         self.logger = logger
 
     def __call__(self, *args, method_name=None, **kwargs):
         """
 
+        :param str method_name: the method to be found in :attr:`available_methods`
         :param str method: the method to be found in :attr:`available_methods`
         :param *args: positional arguments
         :param *kwargs: keyword arguments
@@ -246,14 +295,12 @@ class ConvBase(metaclass=ConvMeta):
         method_name = method_name or self.default
 
         # If not, we need to check the name
-        # "dummy" is a method used to evaluate the cost of the
         # execute() method for the benchmark
-        if method_name not in self.available_methods + ['dummy']:
+        if method_name not in self.available_methods:
             msg = "Methods available are {}".format(self.available_methods)
             _log.error(msg)
             raise ValueError(msg)
 
-
         _log.info("{}> Executing {} method ".format(self.name, method_name))
         # reference to the method requested
         method_reference = getattr(self, "_method_{}".format(method_name))
@@ -261,10 +308,10 @@ class ConvBase(metaclass=ConvMeta):
         # call the method itself
 
         t1 = time.time()
+
         method_reference(*args, **kwargs)
         t2 = time.time()
-        _log.info("Took {} seconds ".format(t2-t1))
-
+        _log.info("Took {} seconds ".format(t2 - t1))
 
     @property
     def name(self):
@@ -273,22 +320,23 @@ class ConvBase(metaclass=ConvMeta):
         """
         return type(self).__name__
 
-    def _method_dummy(self, *args, **kwargs):
-        # The execute commands has a large initialisation cost (about a second)
-        # This commands does not and can be used to evaluate that cost
-        self.execute("")
-
     def shell(self, cmd):
         from bioconvert.core.shell import shell
+
         _log.info("CMD: {}".format(cmd))
         shell(cmd)
 
     def execute(self, cmd, ignore_errors=False, verbose=False, shell=False):
 
+        if ">" in cmd:
+            lhs, rhs = cmd.split(">", 1)
+            cmd = lhs + self._extra_arguments + ">" + rhs
+        else:
+            cmd = cmd + self._extra_arguments
+
         if shell is True or self._execute_mode == "shell":
             self.shell(cmd)
             return
-        _log.info("CMD: {}".format(cmd))
         self._execute(cmd, ignore_errors, verbose)
 
     def _execute(self, cmd, ignore_errors=False, verbose=False):
@@ -305,11 +353,7 @@ class ConvBase(metaclass=ConvMeta):
         :rtype: a :class:`StringIO` instance
         """
         try:
-            process_ = Popen(cmd,
-                             shell=True,
-                             stdout=PIPE,
-                             stderr=PIPE,
-                             stdin=None)
+            process_ = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE, stdin=None)
         except Exception as err:
             msg = "Failed to execute Command: '{}'. error: '{}'".format(cmd, err)
             raise RuntimeError(msg)
@@ -338,6 +382,7 @@ class ConvBase(metaclass=ConvMeta):
                         output.write(data.decode("utf-8"))
                     elif flow is process_.stderr:
                         errors.write(data.decode("utf-8"))
+                        print(process_.stderr)
                 readable, writable, exceptional = select.select(inputs, [], [], 1)
 
         errors = errors.getvalue().strip()
@@ -351,18 +396,29 @@ class ConvBase(metaclass=ConvMeta):
         else:
             return output
 
-    def boxplot_benchmark(self, N=5, rerun=True, include_dummy=False,
-            to_exclude=[], to_include=[], rot_xticks=90, boxplot_args={}):
+    def boxplot_benchmark(
+        self,
+        N=5,
+        rerun=True,
+        to_exclude=[],
+        to_include=[],
+        rot_xticks=90,
+        boxplot_args={},
+    ):
         """Simple wrapper to call :class:`Benchmark` and plot the results
 
         see :class:`~bioconvert.core.benchmark.Benchmark` for details.
 
         """
-        self._benchmark = Benchmark(self, N=N, to_exclude=to_exclude,
-                                    to_include=to_include)
-        self._benchmark.include_dummy = include_dummy
-        data = self._benchmark.plot(rerun=rerun, rot_xticks=rot_xticks,
-                                    boxplot_args=boxplot_args)
+        if to_include == "all":
+            to_include = []
+
+        self._benchmark = Benchmark(
+            self, N=N, to_exclude=to_exclude, to_include=to_include
+        )
+        data = self._benchmark.plot(
+            rerun=rerun, rot_xticks=rot_xticks, boxplot_args=boxplot_args
+        )
         return data
 
     def _get_default_method(self):
@@ -372,6 +428,7 @@ class ConvBase(metaclass=ConvMeta):
             return self.available_methods[0]
         else:
             return self._default_method
+
     default = property(_get_default_method)
 
     def install_tool(self, executable):
@@ -391,26 +448,34 @@ class ConvBase(metaclass=ConvMeta):
             logger.info("Installing tool : " + executable)
             # Assigned but never used, says flake8
             # bioconvert_path = bioconvert.__path__[0]
-            script = bioconvert_data(
-                'install_' + executable + '.sh', where="../misc")
-            subprocess.call(['sh', script])
+            script = bioconvert_data("install_" + executable + ".sh", where="../misc")
+            subprocess.call(["sh", script])
 
     @classmethod
     def add_argument_to_parser(cls, sub_parser):
         sub_parser.description = cls.get_description()
-        for arg in itertools.chain(cls.get_common_arguments_for_converter(), cls.get_additional_arguments()):
+        for arg in itertools.chain(
+            cls.get_IO_arguments(),
+            cls.get_common_arguments_for_converter(),
+            cls.get_additional_arguments(),
+        ):
             arg.add_to_sub_parser(sub_parser)
 
     @classmethod
     def get_description(cls):
-        return "Allow to convert file in '%s' to '%s' format." % ConvMeta.split_converter_to_extensions(cls.__name__)
+        msg = "Convert file from '{}' to '{}' format. "
+        msg += "See bioconvert.readthedocs.io for details"
+        msg = msg.format(*ConvMeta.split_converter_to_format(cls.__name__))
+        return msg
 
     @classmethod
     def get_additional_arguments(cls):
         return []
 
+    # common arguments for the sub command case
+    # when using bioconvert <conversion>
     @staticmethod
-    def get_common_arguments():
+    def get_IO_arguments():
         yield ConvArg(
             names="input_file",
             nargs="?",
@@ -426,71 +491,111 @@ class ConvBase(metaclass=ConvMeta):
             output_argument=True,
             help="The path where the result will be stored.",
         )
+
+    @staticmethod
+    def get_common_arguments():
         yield ConvArg(
-            names=["-f", "--force", ],
+            names=["-f", "--force",],
             action="store_true",
             help="if outfile exists, it is overwritten with this option",
         )
         yield ConvArg(
-            names=["-v", "--verbosity", ],
+            names=["-v", "--verbosity",],
             default=bioconvert.logger.level,
             help="Set the outpout verbosity.",
             choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
         )
         yield ConvArg(
-            names=["--raise-exception", ],
+            names=["--raise-exception",],
             action="store_true",
             help="Let exception ending the execution be raised and displayed",
         )
         yield ConvArg(
-            names=["-m", "--batch", ],
+            names=["-X", "--batch",],
             default=False,
             action="store_true",
             help="Allow conversion of a set of files using wildcards. You "
-                "must use quotes to escape the wildcards. For instance: "
-                "--batch 'test*fastq' ")
+            "must use quotes to escape the wildcards. For instance: "
+            "--batch 'test*fastq' ",
+        )
         yield ConvArg(
-            names=["-b", "--benchmark", ],
+            names=["-b", "--benchmark",],
             default=False,
             action="store_true",
             help="Running all available methods",
         )
         yield ConvArg(
-            names=["-N", "--benchmark-N", ],
+            names=["-N", "--benchmark-N",],
             default=5,
             type=int,
             help="Number of trials for each methods",
         )
         yield ConvArg(
-            names=["-a", "--allow-indirect-conversion", ],
+            names=["-T", "--benchmark-tag",],
+            default="bioconvert",
+            help="Save results (json and image) named after this tag",
+        )
+        yield ConvArg(
+            names=["-I", "--benchmark-save-image",],
+            action="store_true",
+            help="Save results in this image",
+        )
+        yield ConvArg(
+            names=["-M", "--benchmark-methods",],
+            default="all",
+            nargs="+",
+            type=str,
+            help="Methods to include. Provide list as space-separated method names. Use -s  to get the full list.",
+        )
+        yield ConvArg(
+            names=["-a", "--allow-indirect-conversion",],
             default=False,
             action="store_true",
             help="Allow to chain converter when direct conversion is absent",
         )
+        yield ConvArg(
+            names=["-e", "--extra-arguments",],
+            default="",
+            help="Any arguments accepted by the method's tool",
+        )
 
     @classmethod
     def get_common_arguments_for_converter(cls):
         for a in ConvBase.get_common_arguments():
             yield a
         try:
-            # Some converter does not have any method and work in __call__, so preventing to crash by searching for them
+            # Some converters do not have any method and work
+            # in __call__, so preventing to crash by searching for them
             yield ConvArg(
-                names=["-c", "--method", ],
+                names=["-m", "--method",],
                 nargs="?",
                 default=cls._get_default_method(cls),
                 help="The method to use to do the conversion.",
                 choices=cls.available_methods,
             )
         except Exception as e:
-            _log.warning("converter '{}' does not seems to have methods: {}".format(cls.__name__, e))
+            _log.warning(
+                "converter '{}' does not seems to have methods: {}".format(
+                    cls.__name__, e
+                )
+            )
             pass
         yield ConvArg(
-            names=["-s", "--show-methods", ],
+            names=["-s", "--show-methods",],
             default=False,
             action="store_true",
             help="A converter may have several methods",
         )
 
+        if cls._threading:
+            yield ConvArg(
+                names=["-t", "--threads"],
+                # nargs=1,
+                type=int,
+                default=cls.threads,
+                help="threads to be used",
+            )
+
 
 # Implementing a class creator
 # The created class will have the correct name, will inherit from ConvBase
@@ -503,7 +608,7 @@ def make_chain(converter_map):
     """
     in_fmt = converter_map[0][0][0]
     out_fmt = converter_map[-1][0][1]
-    chain_name = "2".join([in_fmt.capitalize(), out_fmt.capitalize()])
+    chain_name = "{}2{}".format("_".join(in_fmt), "_".join(out_fmt))
     chain_attributes = {}
 
     def chain_init(self, infile, outfile):
@@ -520,8 +625,9 @@ def make_chain(converter_map):
 
         # Contains the last temporary output file, if any
         pipe_files = deque()
-        for (step_num, ((_, out_fmt), converter)) \
-                in enumerate(self.converter_map, start=1):
+        for (step_num, ((_, out_fmt), converter)) in enumerate(
+            self.converter_map, start=1
+        ):
             if step_num == 1:
                 # May not be necessary:
                 step_infile = None
@@ -537,9 +643,12 @@ def make_chain(converter_map):
                 step_outfile = None
                 step_output = self.outfile
             else:
-                step_outfile = TempFile(suffix=out_fmt.lower())
-                step_output = step_outfile.name
-                pipe_files.append(step_outfile)
+
+                # FIXME: for mutiple IO converters
+                if len(out_fmt) == 1:
+                    step_outfile = TempFile(suffix=out_fmt[0].lower())
+                    step_output = step_outfile.name
+                    pipe_files.append(step_outfile)
 
             conv_step(converter, step_input, step_output)
             if del_infile:
